@@ -5,10 +5,10 @@ mod meta;
 mod rss;
 mod templating;
 
-use crate::templating::TemplateEngine;
 use anyhow::Result;
 use clap::Parser;
 use gen::generate;
+use log::error;
 use std::path::{Path, PathBuf};
 
 #[derive(Parser)]
@@ -20,17 +20,33 @@ struct Cli {
     #[clap(long, short = 's')]
     /// Path to source directory
     source: Option<PathBuf>,
+    #[clap(long, short = 'v')]
+    /// Verbose output
+    verbose: bool,
 }
 
 fn main() -> Result<()> {
-    env_logger::init_from_env(env_logger::Env::new().default_filter_or("info"));
-
     let cli = Cli::parse();
+
+    let level = {
+        if cli.verbose {
+            log::LevelFilter::Trace
+        } else {
+            log::LevelFilter::Info
+        }
+    };
+    env_logger::builder()
+        .format_module_path(false)
+        .format_timestamp(None)
+        .filter_level(level)
+        .init();
+
     let source_dir = cli.source.unwrap_or(Path::new(".").to_path_buf());
-
-    let template_engine = TemplateEngine::load(source_dir.clone())?;
-
-    generate(&source_dir, &cli.output, &template_engine)?;
+    if source_dir == cli.output {
+        error!("The source directory can't be the destination directory!");
+    } else {
+        generate(&source_dir, &cli.output)?;
+    }
 
     Ok(())
 }
